import logging
from telegram import (
    Update,
    ReplyKeyboardMarkup,
    KeyboardButton,
    InlineKeyboardButton,
    InlineKeyboardMarkup,
)
from telegram.ext import (
    Application,
    CommandHandler,
    MessageHandler,
    filters,  # تغییر Filters به filters
    CallbackContext,
    CallbackQueryHandler,
    ConversationHandler,
)
import os
from decouple import config
import datetime
import sqlite3
from PyPDF2 import PdfWriter
from io import BytesIO
import asyncio

# --- Logging ---
logging.basicConfig(
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
)
logger = logging.getLogger(__name__)

# --- Global Variables ---
(
    JOIN_CHANNEL,
    WELCOME_MESSAGE,
    RULES_CONFIRM,
    PROFILE_INFO,
    POST_TYPE,
    GET_FILE,
    GET_CAPTION,
    CONFIRM_POST,
    CHAT_WITH_USER,
    CREATE_INVOICE,
    STOP_BOT,
    MAIN_MENU,
    SHOW_CARD,
) = range(
    13
)
ADMIN_IDS = [int(config("ADMIN_ID_1")), int(config("ADMIN_ID_2"))]
CHANNEL_ID = config("CHANNEL_ID")
BOT_TOKEN = config("BOT_TOKEN")

DEFAULT_IMAGE_PATH = "images/default.png" # مسیر تصویر پیش فرض
DATABASE_NAME = "bot_database.db" # نام دیتابیس


# --- Database Functions ---
def create_connection():
    conn = None
    try:
        conn = sqlite3.connect(DATABASE_NAME)
        return conn
    except sqlite3.Error as e:
        print(e)
    return conn

def create_tables():
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute(
                """
                CREATE TABLE IF NOT EXISTS users (
                    user_id INTEGER PRIMARY KEY,
                    first_name TEXT,
                    last_name TEXT,
                    business_name TEXT,
                    phone_number TEXT,
                    work_number TEXT,
                    rules_accepted INTEGER,
                    join_date DATETIME
                )
                """
            )
            cursor.execute(
                """
                CREATE TABLE IF NOT EXISTS posts (
                    post_id INTEGER PRIMARY KEY AUTOINCREMENT,
                    user_id INTEGER,
                    post_type TEXT,
                    file_id TEXT,
                    caption TEXT,
                    post_date DATETIME,
                    is_approved INTEGER DEFAULT 0,
                    FOREIGN KEY (user_id) REFERENCES users (user_id)
                )
                """
            )
            cursor.execute(
                """
                    CREATE TABLE IF NOT EXISTS chats (
                        chat_id INTEGER PRIMARY KEY AUTOINCREMENT,
                        user1_id INTEGER,
                        user2_id INTEGER,
                        message TEXT,
                        timestamp DATETIME,
                        FOREIGN KEY (user1_id) REFERENCES users (user_id),
                        FOREIGN KEY (user2_id) REFERENCES users (user_id)
                    )
                """
            )
            conn.commit()
        except sqlite3.Error as e:
            print(e)
        finally:
            conn.close()


def insert_user(user_id, first_name, last_name, business_name, phone_number, work_number):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute(
                """
                INSERT INTO users (user_id, first_name, last_name, business_name, phone_number, work_number, rules_accepted, join_date)
                VALUES (?, ?, ?, ?, ?, ?, 1, ?)
                """,
                (user_id, first_name, last_name, business_name, phone_number, work_number, datetime.datetime.now()),
            )
            conn.commit()
            return True
        except sqlite3.Error as e:
            print(e)
            return False
        finally:
            conn.close()
    return False

def update_rules_status(user_id):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute("UPDATE users SET rules_accepted = 1 WHERE user_id = ?", (user_id,))
            conn.commit()
            return True
        except sqlite3.Error as e:
            print(e)
            return False
        finally:
            conn.close()
    return False


def get_user_data(user_id):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM users WHERE user_id = ?", (user_id,))
            user_data = cursor.fetchone()
            return user_data
        except sqlite3.Error as e:
            print(e)
            return None
        finally:
            conn.close()
    return None

def insert_post(user_id, post_type, file_id, caption):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute(
                "INSERT INTO posts (user_id, post_type, file_id, caption, post_date) VALUES (?, ?, ?, ?, ?)",
                (user_id, post_type, file_id, caption, datetime.datetime.now()),
            )
            conn.commit()
            return True
        except sqlite3.Error as e:
            print(e)
            return False
        finally:
            conn.close()
    return False


def insert_chat_log(user1_id, user2_id, message):
     conn = create_connection()
     if conn:
         try:
             cursor = conn.cursor()
             cursor.execute(
                 "INSERT INTO chats (user1_id, user2_id, message, timestamp) VALUES (?, ?, ?, ?)",
                 (user1_id, user2_id, message, datetime.datetime.now()),
             )
             conn.commit()
             return True
         except sqlite3.Error as e:
             print(e)
             return False
         finally:
            conn.close()
     return False

def get_user_posts(user_id):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM posts WHERE user_id = ?", (user_id,))
            user_posts = cursor.fetchall()
            return user_posts
        except sqlite3.Error as e:
            print(e)
            return None
        finally:
            conn.close()
    return None
def get_post_data(post_id):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM posts WHERE post_id = ?", (post_id,))
            post_data = cursor.fetchone()
            return post_data
        except sqlite3.Error as e:
            print(e)
            return None
        finally:
            conn.close()
    return None
def update_post_approval(post_id, is_approved):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute("UPDATE posts SET is_approved = ? WHERE post_id = ?", (is_approved, post_id))
            conn.commit()
            return True
        except sqlite3.Error as e:
            print(e)
            return False
        finally:
            conn.close()
    return False

def delete_post(post_id):
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute("DELETE FROM posts WHERE post_id = ?", (post_id,))
            conn.commit()
            return True
        except sqlite3.Error as e:
            print(e)
            return False
        finally:
            conn.close()
    return False

def log_rules_agreement(user_id, first_name, last_name, rules_text):
     conn = create_connection()
     if conn:
         try:
             cursor = conn.cursor()
             cursor.execute("INSERT INTO rule_logs (user_id, first_name, last_name, rules_text, timestamp) VALUES (?, ?, ?, ?, ?)", (user_id, first_name, last_name, rules_text, datetime.datetime.now()))
             conn.commit()
             return True
         except sqlite3.Error as e:
             print(e)
             return False
         finally:
             conn.close()
     return False

def get_all_user_ids():
    conn = create_connection()
    if conn:
        try:
            cursor = conn.cursor()
            cursor.execute("SELECT user_id FROM users")
            user_ids = [row[0] for row in cursor.fetchall()]
            return user_ids
        except sqlite3.Error as e:
            print(e)
            return None
        finally:
            conn.close()
    return None

# --- Bot Functions ---
def start(update: Update, context: CallbackContext) -> int:
    user = update.effective_user
    user_id = user.id
    user_data = get_user_data(user_id)
    if user_data and user_data[6] == 1:
         update.message.reply_text(f"به ربات خوش آمدید، {user.first_name}! \n\n چه کاری میتونم برات انجام بدم؟",reply_markup=main_menu_keyboard())
         return MAIN_MENU
    else:
         if user_data:
            update.message.reply_text(
                    "به ربات خوش آمدید! لطفا قوانین را تایید کنید تا بتوانید از ربات استفاده کنید."
            )
            show_rules(update, context)
            return RULES_CONFIRM
         else:
            update.message.reply_text(
                    "به ربات خوش آمدید! لطفا ابتدا در کانال عضو شوید و سپس به ربات بازگردید."
             )
            channel_link = f"https://t.me/{config('CHANNEL_USERNAME')}"
            update.message.reply_text(
                f"لطفا ابتدا در کانال ما عضو شوید: {channel_link} \n و بعد دستور /start را دوباره بزنید"
            )
            return JOIN_CHANNEL


def show_rules(update: Update, context: CallbackContext):
    rules_text = """
    قوانین و مقررات استفاده از ربات:
    1. شما موظف به رعایت ادب و احترام در تعاملات خود هستید.
    2. انتشار محتوای نامناسب، توهین آمیز و یا تبلیغاتی ممنوع است.
    3. مسئولیت هرگونه فعالیت شما در ربات بر عهده خودتان است.
    4. اطلاعات شخصی خود را به طور کامل و صحیح وارد کنید.
    5. در صورت نقض قوانین، دسترسی شما به ربات محدود خواهد شد.
    با تایید قوانین، شما موافقت خود را با این شرایط اعلام می‌کنید.
    """

    keyboard = [
        [
            InlineKeyboardButton("بله، موافقم", callback_data="rules_accept"),
            InlineKeyboardButton("خیر، نمی‌پذیرم", callback_data="rules_reject"),
        ]
    ]
    reply_markup = InlineKeyboardMarkup(keyboard)
    update.message.reply_text(rules_text, reply_markup=reply_markup)

def handle_rules_confirmation(update: Update, context: CallbackContext):
    query = update.callback_query
    query.answer()
    user_id = query.from_user.id
    first_name = query.from_user.first_name
    last_name = query.from_user.last_name

    if query.data == "rules_accept":
        update_rules_status(user_id)
        rules_text = """
            قوانین و مقررات استفاده از ربات:
            1. شما موظف به رعایت ادب و احترام در تعاملات خود هستید.
            2. انتشار محتوای نامناسب، توهین آمیز و یا تبلیغاتی ممنوع است.
            3. مسئولیت هرگونه فعالیت شما در ربات بر عهده خودتان است.
            4. اطلاعات شخصی خود را به طور کامل و صحیح وارد کنید.
            5. در صورت نقض قوانین، دسترسی شما به ربات محدود خواهد شد.
            با تایید قوانین، شما موافقت خود را با این شرایط اعلام می‌کنید.
        """
        log_rules_agreement(user_id,first_name,last_name, rules_text )
        query.edit_message_text(
            "شما قوانین را پذیرفتید. لطفا اطلاعات پروفایل خود را تکمیل کنید.",
            reply_markup=None,
        )
        query.message.reply_text("لطفا نام و نام خانوادگی خود را وارد کنید:")
        return PROFILE_INFO
    elif query.data == "rules_reject":
        query.edit_message_text(
            "متاسفم، شما باید قوانین را بپذیرید تا بتوانید از ربات استفاده کنید.",
            reply_markup=None,
        )
        return ConversationHandler.END

def get_profile_info(update: Update, context: CallbackContext):
    user_id = update.effective_user.id
    text = update.message.text
    if "profile_data" not in context.user_data:
         context.user_data["profile_data"] = {}
    if "name" not in context.user_data["profile_data"]:
        context.user_data["profile_data"]["name"] = text
        update.message.reply_text("لطفا نام کسب و کار خود را وارد کنید:")
        return PROFILE_INFO
    elif "business_name" not in context.user_data["profile_data"]:
        context.user_data["profile_data"]["business_name"] = text
        update.message.reply_text("لطفا شماره موبایل خود را وارد کنید:")
        return PROFILE_INFO
    elif "phone_number" not in context.user_data["profile_data"]:
        context.user_data["profile_data"]["phone_number"] = text
        update.message.reply_text("لطفا تلفن محل کار خود را وارد کنید:")
        return PROFILE_INFO
    elif "work_number" not in context.user_data["profile_data"]:
        context.user_data["profile_data"]["work_number"] = text
        profile_data = context.user_data["profile_data"]
        first_name, last_name = profile_data["name"].split(" ", 1) if " " in profile_data["name"] else (profile_data["name"], "")
        business_name = profile_data["business_name"]
        phone_number = profile_data["phone_number"]
        work_number = profile_data["work_number"]

        if insert_user(
                user_id,
                first_name,
                last_name,
                business_name,
                phone_number,
                work_number,
        ):
            update.message.reply_text("اطلاعات شما با موفقیت ثبت شد.",reply_markup=main_menu_keyboard())
            context.user_data.clear()
            return MAIN_MENU
        else:
            update.message.reply_text("خطا در ثبت اطلاعات. لطفا دوباره تلاش کنید.")
            return PROFILE_INFO
    return ConversationHandler.END

def create_post_button(update: Update, context: CallbackContext):
    keyboard = [
        [
            InlineKeyboardButton(
                "پیشنهاد فروش پارچه", callback_data="post_type_sell"
            ),
            InlineKeyboardButton(
                "درخواست خرید پارچه", callback_data="post_type_buy"
            ),
        ]
    ]
    reply_markup = InlineKeyboardMarkup(keyboard)
    update.message.reply_text("لطفا نوع پست خود را انتخاب کنید:", reply_markup=reply_markup)
    return POST_TYPE

def handle_post_type_selection(update: Update, context: CallbackContext):
    query = update.callback_query
    query.answer()
    context.user_data["post_type"] = query.data.split("_")[2]
    query.edit_message_text(
        "لطفا عکس یا ویدیو خود را ارسال کنید. اگر نمی خواهید فایل ارسال کنید فقط یک متن بنویسید", reply_markup=None
    )
    return GET_FILE

def get_post_file(update: Update, context: CallbackContext):
     if update.message.photo:
        file = update.message.photo[-1].get_file()
        context.user_data["file_id"] = file.file_id
     elif update.message.video:
         file = update.message.video.get_file()
         context.user_data["file_id"] = file.file_id
     else:
        context.user_data["file_id"] = None
     update.message.reply_text("لطفا کپشن پست خود را وارد کنید:")
     return GET_CAPTION

def get_post_caption(update: Update, context: CallbackContext):
    caption = update.message.text
    context.user_data["caption"] = caption
    post_type = context.user_data["post_type"]
    file_id = context.user_data.get("file_id", None)
    formatted_caption = f"<b>{post_type.capitalize()}:</b>\n{caption}"
    keyboard = [
        [
            InlineKeyboardButton("تایید", callback_data="confirm_post"),
            InlineKeyboardButton("ویرایش", callback_data="edit_post"),
        ]
    ]
    reply_markup = InlineKeyboardMarkup(keyboard)
    if file_id:
         if update.message.photo:
              update.message.reply_photo(photo = file_id, caption=formatted_caption, reply_markup = reply_markup, parse_mode = "HTML")
         elif update.message.video:
              update.message.reply_video(video = file_id, caption=formatted_caption, reply_markup = reply_markup, parse_mode = "HTML")
    else:
         update.message.reply_text(formatted_caption, reply_markup=reply_markup, parse_mode="HTML")
    return CONFIRM_POST

def handle_post_confirmation(update: Update, context: CallbackContext):
     query = update.callback_query
     query.answer()

     if query.data == "confirm_post":
        user_id = query.from_user.id
        post_type = context.user_data["post_type"]
        file_id = context.user_data.get("file_id", None)
        caption = context.user_data["caption"]

        if insert_post(user_id, post_type, file_id, caption):
          if file_id:
              if query.message.photo:
                context.bot.copy_message(chat_id= CHANNEL_ID, message_id= query.message.message_id,from_chat_id= query.message.chat.id, reply_markup=post_buttons(user_id, context))
              elif query.message.video:
                context.bot.copy_message(chat_id= CHANNEL_ID, message_id= query.message.message_id,from_chat_id= query.message.chat.id, reply_markup=post_buttons(user_id, context))

          else:
               formatted_caption = f"<b>{post_type.capitalize()}:</b>\n{caption}"
               context.bot.send_message(chat_id = CHANNEL_ID, text = formatted_caption, reply_markup=post_buttons(user_id, context), parse_mode = "HTML")

          query.edit_message_text("پست شما با موفقیت به کانال ارسال شد.", reply_markup=None)
          context.user_data.clear()
          return ConversationHandler.END
        else:
             query.edit_message_text("خطا در ارسال پست. لطفا دوباره تلاش کنید.", reply_markup=None)
             return GET_CAPTION

     elif query.data == "edit_post":
          query.edit_message_text( "لطفا کپشن جدید خود را وارد کنید:", reply_markup=None )
          return GET_CAPTION

def post_buttons(user_id, context: CallbackContext):
    user_data = get_user_data(user_id)
    if user_data:
      keyboard = [
        [
              InlineKeyboardButton("چت", callback_data=f"chat_{user_id}"),
              InlineKeyboardButton("مشاهده کارت ویزیت", callback_data=f"card_{user_id}")
            ]
      ]
      if user_id in ADMIN_IDS:
        keyboard[0].extend([
          InlineKeyboardButton("مجاز", callback_data=f"approve_post"),
          InlineKeyboardButton("حذف پست", callback_data=f"delete_post")
        ])
      return InlineKeyboardMarkup(keyboard)

    else:
         return None

def handle_post_actions(update: Update, context: CallbackContext):
    query = update.callback_query
    query.answer()
    data = query.data.split("_")
    action = data[0]

    if action == "chat":
        user_id = query.from_user.id
        target_user_id = int(data[1])
        context.user_data["target_user_id"] = target_user_id

        query.message.reply_text("شما وارد محیط چت شدید. پیام خود را بفرستید:")
        return CHAT_WITH_USER
    elif action == "card":
        user_id = int(data[1])
        show_card(update, context, user_id)
    elif action == "approve":
         post_id = context.bot_data["post_id"]
         if update_post_approval(post_id, 1):
            query.edit_message_text("پست با موفقیت مجاز شد", reply_markup=None)
         else:
           query.edit_message_text("خطا در تایید پست", reply_markup=None)
    elif action == "delete":
          post_id = context.bot_data["post_id"]
          if delete_post(post_id):
             query.edit_message_text("پست با موفقیت حذف شد", reply_markup=None)
             context.bot.delete_message(chat_id=query.message.chat_id,message_id=query.message.message_id)
          else:
            query.edit_message_text("خطا در حذف پست", reply_markup=None)

def chat_with_user(update: Update, context: CallbackContext):
    message = update.message.text
    user_id = update.effective_user.id
    target_user_id = context.user_data.get("target_user_id", None)

    if target_user_id:
      if insert_chat_log(user_id, target_user_id, message):
        context.bot.send_message(chat_id=target_user_id,text=f"پیام جدید از {update.effective_user.first_name} : \n {message}" )
        update.message.reply_text(f"پیام شما به {get_user_data(target_user_id)[1]} با موفقیت ارسال شد.")
        return MAIN_MENU
      else:
          update.message.reply_text("خطا در ارسال پیام. لطفا دوباره تلاش کنید.")
          return CHAT_WITH_USER
    else:
      update.message.reply_text("خطا در ارسال پیام. لطفا دوباره تلاش کنید.")
      return MAIN_MENU

def generate_card(user_data):
     name = f"{user_data[1]} {user_data[2]}"
     business_name = user_data[3]
     phone_number = user_data[4]
     work_number = user_data[5]
     card_text = f"""
     نام: {name}
     نام کسب و کار: {business_name}
     موبایل: {phone_number}
     تلفن محل کار: {work_number}
     """
     return card_text

def show_card(update: Update, context: CallbackContext, user_id):
    user_data = get_user_data(user_id)
    if user_data:
      card_text = generate_card(user_data)
      update.callback_query.message.reply_text(card_text)
    else:
        update.callback_query.message.reply_text("کاربری یافت نشد.")


def create_invoice_button(update: Update, context: CallbackContext):
    keyboard = [
        [
            InlineKeyboardButton("صدور پیش فاکتور", callback_data="generate_invoice"),
        ]
    ]
    reply_markup = InlineKeyboardMarkup(keyboard)
    update.message.reply_text("برای صدور پیش فاکتور دکمه زیر را بزنید:", reply_markup=reply_markup)
    return CREATE_INVOICE

def handle_invoice_generation(update: Update, context: CallbackContext):
    query = update.callback_query
    query.answer()
    user_id = query.from_user.id
    user_data = get_user_data(user_id)
    if user_data:
        invoice_text = generate_card(user_data)
        pdf_file = create_pdf_invoice(invoice_text)
        query.message.reply_document(document = pdf_file,filename = "invoice.pdf" )
    else:
        query.message.reply_text("کاربری یافت نشد.")
    return ConversationHandler.END

def create_pdf_invoice(text):
     pdf_buffer = BytesIO()
     pdf_writer = PdfWriter()
     pdf_writer.add_blank_page()
     pdf_writer.add_metadata({'Title': 'Invoice', 'Author': "Bot"})
     pdf_content = f"پیش فاکتور\n\n {text}"
     page = pdf_writer.pages[0]
     page.insert_content(pdf_content.encode("utf-8"))
     pdf_writer.write(pdf_buffer)
     pdf_buffer.seek(0)
     return pdf_buffer

def main_menu_keyboard():
     keyboard = [
          [KeyboardButton("ایجاد پست")],
          [KeyboardButton("صدور پیش فاکتور")]
          ]
     return ReplyKeyboardMarkup(keyboard, resize_keyboard=True)

async def main_menu(update: Update, context: CallbackContext):
    text = update.message.text

    if text == "ایجاد پست":
       return create_post_button(update, context)
    elif text == "صدور پیش فاکتور":
       return create_invoice_button(update, context)
    else:
       update.message.reply_text("متوجه نشدم! لطفا یکی از گزینه های زیر را انتخاب کنید.", reply_markup=main_menu_keyboard())
       return MAIN_MENU

def stop_bot(update: Update, context: CallbackContext):
    user_id = update.effective_user.id
    if user_id in ADMIN_IDS:
      context.bot.send_message(
                chat_id=update.effective_chat.id, text="ربات متوقف شد."
      )
      updater.stop()
    else:
        update.message.reply_text("شما دسترسی لازم را ندارید")

def error_handler(update: Update, context: CallbackContext):
    logger.error(f"Update: {update} \nError: {context.error}")
    if update:
        update.message.reply_text(
            "متاسفم، مشکلی پیش آمده است. لطفا دوباره تلاش کنید."
        )

def log_posts_of_user(update: Update, context: CallbackContext):
    user_id = update.effective_user.id
    posts = get_user_posts(user_id)
    if posts:
       log_message = f"پست های کاربر {user_id}:\n"
       for post in posts:
          log_message += f"شناسه پست : {post[0]}, نوع پست: {post[2]}, کپشن : {post[4]}, تاریخ : {post[5]}\n"
       update.message.reply_text(log_message)
    else :
        update.message.reply_text("پستی یافت نشد.")

def set_post_id(update: Update, context: CallbackContext):
    query = update.callback_query
    query.answer()
    post_id = int(query.message.text.split("شناسه پست :")[1].split(",")[0]) if query.message.text and "شناسه پست :" in query.message.text else None
    if post_id:
       context.bot_data["post_id"] = post_id
       handle_post_actions(update, context)
    else:
        query.message.reply_text("پست نامعتبر است.")

def broadcast_message(update: Update, context: CallbackContext):
    user_id = update.effective_user.id
    if user_id in ADMIN_IDS:
        text_to_send = update.message.text.split("/broadcast ")[1]
        all_user_ids = get_all_user_ids()
        if all_user_ids:
            for user_id in all_user_ids:
                try:
                   context.bot.send_message(chat_id = user_id, text = text_to_send)
                except Exception as e:
                    print(f"Could not send to {user_id}, Error: {e}")
            update.message.reply_text("پیام همگانی ارسال شد.")
        else:
             update.message.reply_text("کاربری یافت نشد.")
    else:
        update.message.reply_text("شما دسترسی لازم را ندارید")

def check_post_in_channel(update: Update, context: CallbackContext):
    if update.channel_post:
         message = update.channel_post
         post_id = int(message.text.split("شناسه پست :")[1].split(",")[0]) if message.text and "شناسه پست :" in message.text else None
         if post_id:
           post_data = get_post_data(post_id)
           if post_data[0]:
                context.bot_data["post_id"] = post_id
                context.bot.edit_message_reply_markup(chat_id=message.chat.id, message_id=message.message_id,reply_markup=post_buttons(post_data[1],context))

async def main():
    create_tables()
    application = Application.builder().token(BOT_TOKEN).build()


    conv_handler = ConversationHandler(
        entry_points=[CommandHandler("start", start)],
        states={
            JOIN_CHANNEL: [],
            WELCOME_MESSAGE: [],
            RULES_CONFIRM: [
                CallbackQueryHandler(handle_rules_confirmation),
            ],
            PROFILE_INFO: [MessageHandler(filters.TEXT & ~filters.COMMAND, get_profile_info)],
            MAIN_MENU: [MessageHandler(filters.TEXT & ~filters.COMMAND, main_menu, )],
            POST_TYPE: [CallbackQueryHandler(handle_post_type_selection)],
            GET_FILE: [
                MessageHandler(
                    filters.photo | filters.video | filters.text & ~filters.command, get_post_file
                )
            ],
            GET_CAPTION: [
                MessageHandler(filters.text & ~filters.command, get_post_caption)
            ],
            CONFIRM_POST: [CallbackQueryHandler(handle_post_confirmation)],
            CHAT_WITH_USER: [                MessageHandler(filters.text & ~filters.command, chat_with_user)],
            CREATE_INVOICE: [CallbackQueryHandler(handle_invoice_generation)],
            SHOW_CARD: [],
        },
        fallbacks=[CommandHandler("stop", stop_bot)],
    )
    application.add_handler(conv_handler)
    application.add_handler(MessageHandler(filters.regex("شناسه پست :"),set_post_id))
    application.add_handler(CommandHandler("posts", log_posts_of_user))
    application.add_handler(CommandHandler("broadcast", broadcast_message))
    application.add_handler(MessageHandler(filters.all,check_post_in_channel ))
    application.add_error_handler(error_handler)

    await application.run_polling()

if __name__ == "__main__":
    import asyncio
    asyncio.run(main())